from ldap3 import Connection, ServerPool
from django.conf import settings

def ldap_auth(username, password):
    """
    ldap验证,服务器池setting中设置
    """
    # ladp的服务器池
    ldap_server_pool = ServerPool(settings.LDAP_SERVER_POOL)
    # ad用户处理
    if "/" in username:
        ad_user = username
    elif "@" in username:
        ad_user = username
    else:
        # 如果不符合域用户格式，自动添加域名
        ad_user = f"""{username}@{settings.AD_DOMAIN_NAME}"""
    # 尝试链接域控
    conn = Connection(ldap_server_pool, user=ad_user, password=password,
                      check_names=True, lazy=False, raise_exceptions=False)
    conn.open()
    conn.bind(read_server_info=False)

    is_pass= True if conn.result["result"]==0 else False
    conn.unbind()
    return is_pass

if __name__ == "__main__":
    username = input("请输入用户名:")
    password = input("请输入密码:")
    print(ldap_auth(username, password))
    input("请输入任意键退出:")



from django.contrib import admin
from django.contrib.auth.admin import UserAdmin
from .models import User


class EXUserAdmin(UserAdmin):
    """
    增加了mobile,是否域用户的显示
    """
    fieldsets = (
        (None, {'fields': ('username', 'password')}),
        (('Personal info'), {
         'fields': ('first_name', 'last_name', 'email', 'mobile')}),
        (('Permissions'), {
         'fields': ('is_active', 'is_ad_account', 'is_staff', 'is_superuser', 'groups', 'user_permissions'), }),
        (('Important dates'), {'fields': ('last_login', 'date_joined')}),
    )


admin.site.register(User, EXUserAdmin)



from datetime import timedelta
from django.utils import timezone
from django.conf import settings
from django.db import connection,models
from rest_framework.authtoken.views import ObtainAuthToken
from rest_framework import viewsets
from rest_framework import status
from rest_framework.authtoken.models import Token
from rest_framework.response import Response
from django_filters.rest_framework import DjangoFilterBackend
from rest_framework.filters import SearchFilter,OrderingFilter
from rest_framework.authentication import SessionAuthentication, BasicAuthentication
from .permissions import DjangoModelPermissions
from .authentication import ExpiringTokenAuthentication
from rest_framework.views import APIView
from .serializer import User,User_Serializer
import json
from django_restful_response import Restful_Response
from .models import OperationLog
# from asset.views_ud import Restful_Response





# Create your views here.
class ExpiringObtainAuthToken(ObtainAuthToken):
    """
    登录获取token
    """
    def post(self, request, *args, **kwargs):  
        serializer = self.get_serializer(data=request.data)
        try:
            serializer.is_valid(raise_exception=True)
            login_succeed=True
        except:
            login_succeed=False
        if login_succeed:
            user = serializer.validated_data['user']
            token, created = Token.objects.get_or_create(user=user)
            #如果超过设置的3天的，那么更新token并重新获取token
            if timezone.now() > (token.created + timedelta(seconds=settings.SESSION_COOKIE_AGE)):
                Token.objects.filter(user=user).update(key=Token.generate_key(), created=timezone.now())
                token=Token.objects.get(user=user)
            return Response({'token':token.key},status=status.HTTP_200_OK) 
        else:
            # 如果错误返回401
            return Response({'token':""},status=status.HTTP_401_UNAUTHORIZED) 

obtain_auth_token = ExpiringObtainAuthToken.as_view()


class User_Set(viewsets.ModelViewSet):
    authentication_classes = [ExpiringTokenAuthentication, SessionAuthentication,BasicAuthentication]
    permission_classes = [DjangoModelPermissions]
    filter_backends = (SearchFilter, DjangoFilterBackend)
    queryset = User.objects.all()
    serializer_class = User_Serializer
    search_fields = ["username","first_name"]
    filterset_fields = []
    def list(self, request, *args, **kwargs):
        queryset = self.filter_queryset(self.get_queryset())
        page = self.paginate_queryset(queryset)
        if page is not None:
            serializer = self.get_serializer(page, many=True)
            return self.get_paginated_response(serializer.data)
        serializer = self.get_serializer(queryset, many=True)
        return Response(serializer.data)



class User_Right_Set(APIView):
    authentication_classes = [ExpiringTokenAuthentication, SessionAuthentication,BasicAuthentication]
    def get(self, request, format=None):
        condition_list=[f"""v1.username='{request.user.username}' """]
        if "app_label" in request.GET and bool(request.GET["app_label"]):
            condition_list.append(f"""v4.app_label='{request.GET["app_label"]}'""")
        if "model" in request.GET and bool(request.GET["model"]):
            condition_list.append(f"""v4.model='{request.GET["model"]}'""")
        sql_str=f"""  SELECT v3.name,v4.model,v3.codename
  FROM [dbo].[auth_user] v1
  inner join [dbo].[auth_user_user_permissions] v2 on v1.id=v2.user_id
  inner join [dbo].[auth_permission] v3 on v2.permission_id=v3.id
  inner join [dbo].[django_content_type] v4 on v3.content_type_id=v4.id                 
  where {"and ".join(condition_list)}
  union 
  SELECT v3.name,v4.model,v3.codename
  FROM [dbo].[auth_user] v1
  inner join [dbo].[auth_user_groups] v1g on v1.id=v1g.user_id
  inner join [dbo].[auth_group_permissions] v2 on v1g.group_id=v2.group_id
  inner join [dbo].[auth_permission] v3 on v2.permission_id=v3.id
  inner join [dbo].[django_content_type] v4 on v3.content_type_id=v4.id
  where {"and ".join(condition_list)}
  union 
  select v3.name,v4.model,v3.codename
  from [dbo].[auth_user] v1
  inner join [dbo].[auth_permission] v3 on 1=1
  inner join [dbo].[django_content_type] v4 on v3.content_type_id=v4.id
  where {"and ".join(condition_list)} and v1.is_superuser=1
        """
        with connection.cursor() as cursor:
            cursor.execute(sql_str)
            data_list= [row[2] for row in cursor.fetchall()]
        return Response(data_list)

def api_view_logger(permission=None,is_log=True):
    def wrapper(func):
        def deco(*args, **kwargs):
            if permission is None or args[1].user.has_perm(permission):
                # 真正执行函数的地方
                error_code=""
                try:
                    return func(*args, **kwargs)
                except Exception as error:
                    error_code=str(error)
                    return Restful_Response({"message":error_code},status=status.HTTP_500_INTERNAL_SERVER_ERROR)
                finally:
                    if bool(is_log) or bool(error_code):
                        log_obj = OperationLog(qual_name=func.__qualname__,user_id=args[1].user.username,opeart_contect=json.dumps(dict(args[1].data)),error_code=error_code)
                        log_obj.save()
            else:
                return Restful_Response({"message":"验证失败"},status=status.HTTP_401_UNAUTHORIZED)       
        return deco
    return wrapper


from rest_framework import serializers
from rest_framework.relations import RelatedField
from .models import User


class User_Serializer(serializers.ModelSerializer):
    class Meta:
        model = User
        fields = '__all__'




from datetime import timedelta
from rest_framework.authentication import TokenAuthentication
from rest_framework import  exceptions
from django.utils import timezone
from django.conf import settings

class ExpiringTokenAuthentication(TokenAuthentication):
    """
    扩展Token验证是增加过期时间的判断
    """
    def authenticate_credentials(self, key):
        model = self.get_model()
        try:
            token = model.objects.select_related('user').get(key=key)
        except model.DoesNotExist:
            raise exceptions.AuthenticationFailed(('Invalid token.'))
        if not token.user.is_active:
            raise exceptions.AuthenticationFailed(('User inactive or deleted.'))
        elif timezone.now() > (token.created + timedelta(seconds=settings.SESSION_COOKIE_AGE)):
            #如果当前的时间大于Token创建时间+7天，那么久返回Token已经过期
            raise exceptions.AuthenticationFailed(('Token has expired'))
        return (token.user, token)



from django.contrib.auth.backends import ModelBackend
from django.contrib.auth.backends import UserModel
from .ad_auth import ldap_auth

class UserBackend(ModelBackend):
    def authenticate(self, request, username=None, password=None, **kwargs):
        if username is None:
            username = kwargs.get(UserModel.USERNAME_FIELD)
        if username is None or password is None:
            return
        try:
            user = UserModel._default_manager.get_by_natural_key(username)
        except UserModel.DoesNotExist:
            # Run the default password hasher once to reduce the timing
            # difference between an existing and a nonexistent user (#20760).
            UserModel().set_password(password)
        else:
            if user.is_ad_account:
                if ldap_auth(user.username,password) and self.user_can_authenticate(user):
                    # print("域用户登陆成功",user.username)
                    return user
            else: 
                if user.check_password(password) and self.user_can_authenticate(user):
                    # print("网站账号登陆成功",user.username)
                    return user



from rest_framework.permissions import BasePermission
from rest_framework import HTTP_HEADER_ENCODING, exceptions

class DjangoModelPermissions(BasePermission):
    """
    The request is authenticated using `django.contrib.auth` permissions.
    See: https://docs.djangoproject.com/en/dev/topics/auth/#permissions

    It ensures that the user is authenticated, and has the appropriate
    `add`/`change`/`delete` permissions on the model.

    This permission can only be applied against view classes that
    provide a `.queryset` attribute.
    """

    # Map methods into required permission codes.
    # Override this if you need to also provide 'view' permissions,
    # or if you want to provide custom permission codes.
    perms_map = {
        'GET': ['%(app_label)s.view_%(model_name)s'],
        'OPTIONS': [],
        'HEAD': [],
        'POST': ['%(app_label)s.add_%(model_name)s'],
        'PUT': ['%(app_label)s.change_%(model_name)s'],
        'PATCH': ['%(app_label)s.change_%(model_name)s'],
        'DELETE': ['%(app_label)s.delete_%(model_name)s'],
    }

    authenticated_users_only = True

    def get_required_permissions(self, method, model_cls):
        """
        Given a model and an HTTP method, return the list of permission
        codes that the user is required to have.
        """
        kwargs = {
            'app_label': model_cls._meta.app_label,
            'model_name': model_cls._meta.model_name
        }

        if method not in self.perms_map:
            raise exceptions.MethodNotAllowed(method)

        return [perm % kwargs for perm in self.perms_map[method]]

    def _queryset(self, view):
        assert hasattr(view, 'get_queryset') \
            or getattr(view, 'queryset', None) is not None, (
            'Cannot apply {} on a view that does not set '
            '`.queryset` or have a `.get_queryset()` method.'
        ).format(self.__class__.__name__)

        if hasattr(view, 'get_queryset'):
            queryset = view.get_queryset()
            assert queryset is not None, (
                '{}.get_queryset() returned None'.format(view.__class__.__name__)
            )
            return queryset
        return view.queryset

    def has_permission(self, request, view):
        # Workaround to ensure DjangoModelPermissions are not applied
        # to the root view when using DefaultRouter.
        if getattr(view, '_ignore_model_permissions', False):
            return True

        if not request.user or (
           not request.user.is_authenticated and self.authenticated_users_only):
            return False

        queryset = self._queryset(view)
        perms = self.get_required_permissions(request.method, queryset.model)

        return request.user.has_perms(perms)



from setuptools import setup, find_packages  

# with open("tencent_message\README.md", "r",encoding="utf-8") as fh:
#     long_description = fh.read()
long_description="""
0.1.1根据django rest framework的框架的status，优化异常时返回的结构\n
0.1.2 增加message截断功能增加安全性
"""
setup(  
    name = 'django_restful_response',  
    version = '0.1.2',
    # keywords = ('chinesename',),  
    description = 'django rest_framework 框架返回的Respone的异常不符合规范、增加error和message的描述',  
    long_description=long_description,
    long_description_content_type="text/markdown",
    license = 'MIT tencent_message',  
    install_requires = ["django","djangorestframework"],  
    packages = ['django_restful_response'],  # 要打包的项目文件夹
    include_package_data=True,   # 自动打包文件夹内所有数据
    author = 'evanyang',  
    author_email = 'lightyiyi@qq.com',
    url = 'https://www.cnblogs.com/Evan-fanfan/',
    # packages = find_packages(include=("*"),),  
)  


# python setup.py check #检查写的有没有问题，有问题就直接报错了
# python setup.py sdist upload #压缩、并打包上传到pip源上，会在当前目录下产生一个dist文件夹
# #里面就是打好的压缩包
 
# python setup.py sdist #只压缩不上传，就是打成tar.gz这种安装包，给别人用的话直接给他安装包就可以了


from django.db import models
# Create your models here.
from django.contrib.auth.models import AbstractUser


class User(AbstractUser):
    """
    Users within the Django authentication system are represented by this
    model.
    Username and password are required. Other fields are optional.
    """
    mobile = models.CharField('手机号',null=True,blank=True, max_length=14)
    is_ad_account = models.BooleanField('启用域用户登录', default=False)
    def __str__(self):
        return self.username+'-'+self.first_name
    class Meta:
        verbose_name = '用户信息'
        verbose_name_plural = '用户信息'
        db_table = "auth_user"



class OperationLog(models.Model):
    qual_name=models.CharField(max_length=200)
    user_id=models.CharField(max_length=200)
    operated=models.DateTimeField(auto_now=True,null=True)
    opeart_contect=models.CharField(max_length=4000,null=True)
    error_code=models.CharField(max_length=4000,null=True)
    class Meta:
        verbose_name = '操作日志'
        verbose_name_plural = '操作日志'
        db_table = "Log_Operation"
